#include "user_config.h"
#include "user_event.h"
#include "user_gpio.h"
#include "user_player.h"
#include "user_pwm.h"
#include "user_timer.h"
#include "user_uart.h"

#define TAG "auto_gpio"

#define UART_SEND_MAX      16

typedef struct {
  char  data[UART_SEND_MAX];
  int   len;
}uart_data_t;

const uart_data_t g_uart_buf[] = {
  {{0xCF, 0xBB, 0xAF}, 3}, //SerialLatchON
  {{0xFC, 0xED, 0xDA}, 3}, //SerialLatchOFF
  {{0xAA, 0xBB, 0xCC}, 3}, //RobotFWD
  {{0xA1, 0xB2, 0xB3}, 3}, //RobotREV
  {{0xA2, 0xB3, 0xC3}, 3}, //RobotL
  {{0xA3, 0xB4, 0xC5}, 3}, //RobotR
  {{0xA5, 0xB5, 0xC6}, 3}, //RobotStop
  {{0xA4, 0xB6, 0xC7}, 3}, //RobotBack
  {{0x01, 0x10, 0x11}, 3}, //OV
  {{0x11, 0x10, 0x11}, 3}, //CV
  {{0x11, 0x10, 0x00}, 3}, //ASP
  {{0x11, 0x11, 0x11}, 3}, //DSP
  {{0x01, 0x01, 0x01}, 3}, //AS
  {{0x10, 0x10, 0x10}, 3}, //DS
};

static void _custom_setting_cb(USER_EVENT_TYPE event,
                               user_event_context_t *context) {
  event_custom_setting_t *setting = NULL;
  if (context) {
    setting = &context->custom_setting;
    LOGT(TAG, "user command: %s", setting->cmd);
    if (0 == uni_strcmp(setting->cmd, "LightsON")) {
      user_gpio_set_value(GPIO_NUM_A25, 0);
    } else if (0 == uni_strcmp(setting->cmd, "LightsOFF")) {
      user_gpio_set_value(GPIO_NUM_A25, 1);
    } else if (0 == uni_strcmp(setting->cmd, "DelayOn")) {
      user_gpio_set_value(GPIO_NUM_A25, 1);
    } else if (0 == uni_strcmp(setting->cmd, "DelayOff")) {
      user_gpio_set_value(GPIO_NUM_A25, 0);
    } else if (0 == uni_strcmp(setting->cmd, "TurnON")) {
      user_gpio_set_value(GPIO_NUM_A26, 0);
    } else if (0 == uni_strcmp(setting->cmd, "TurnOFF")) {
      user_gpio_set_value(GPIO_NUM_A26, 1);
    } else if (0 == uni_strcmp(setting->cmd, "TurnONCool")) {
      user_gpio_set_value(GPIO_NUM_B2, 1);
    } else if (0 == uni_strcmp(setting->cmd, "TurnOffCool")) {
      user_gpio_set_value(GPIO_NUM_B2, 1);
    } else if (0 == uni_strcmp(setting->cmd, "TurnONHeat")) {
      user_gpio_set_value(GPIO_NUM_B6, 1);
    } else if (0 == uni_strcmp(setting->cmd, "TurnOFFHeat")) {
      user_gpio_set_value(GPIO_NUM_B6, 0);
    } else if (0 == uni_strcmp(setting->cmd, "TurnOnAlarm")) {
      user_gpio_set_value(GPIO_NUM_B7, 1);
    } else if (0 == uni_strcmp(setting->cmd, "TurnOffAlarm")) {
      user_gpio_set_value(GPIO_NUM_B7, 0);
    } else if (0 == uni_strcmp(setting->cmd, "TurnOnDoor")) {
      user_gpio_set_value(GPIO_NUM_A28, 0);
    } else if (0 == uni_strcmp(setting->cmd, "TurnOffDoor")) {
      user_gpio_set_value(GPIO_NUM_A28, 1);
    } else if (0 == uni_strcmp(setting->cmd, "Latchon")) {
      user_gpio_set_value(GPIO_NUM_A28, 1);
    } else if (0 == uni_strcmp(setting->cmd, "Latchoff")) {
      user_gpio_set_value(GPIO_NUM_A28, 0);
    } else if (0 == uni_strcmp(setting->cmd, "SerialLatchON")) {
      user_uart_send(g_uart_buf[0].data, g_uart_buf[0].len);
    } else if (0 == uni_strcmp(setting->cmd, "SerialLatchOFF")) {
      user_uart_send(g_uart_buf[1].data, g_uart_buf[1].len);
    } else if (0 == uni_strcmp(setting->cmd, "OpenGate")) {
      user_gpio_set_value(GPIO_NUM_A28, 0);
    } else if (0 == uni_strcmp(setting->cmd, "SwitchON")) {
      user_gpio_set_value(GPIO_NUM_B7, 0);
    } else if (0 == uni_strcmp(setting->cmd, "SwitchOFF")) {
      user_gpio_set_value(GPIO_NUM_B7, 1);
    } else if (0 == uni_strcmp(setting->cmd, "RobotFWD")) {
      user_uart_send(g_uart_buf[2].data, g_uart_buf[2].len);
    } else if (0 == uni_strcmp(setting->cmd, "RobotREV")) {
      user_uart_send(g_uart_buf[3].data, g_uart_buf[3].len);
    } else if (0 == uni_strcmp(setting->cmd, "RobotL")) {
      user_uart_send(g_uart_buf[4].data, g_uart_buf[4].len);
    } else if (0 == uni_strcmp(setting->cmd, "RobotR")) {
      user_uart_send(g_uart_buf[5].data, g_uart_buf[5].len);
    } else if (0 == uni_strcmp(setting->cmd, "RobotStop")) {
      user_uart_send(g_uart_buf[6].data, g_uart_buf[6].len);
    } else if (0 == uni_strcmp(setting->cmd, "RobotBack")) {
      user_uart_send(g_uart_buf[7].data, g_uart_buf[7].len);
    } else if (0 == uni_strcmp(setting->cmd, "SelfDestruct")) {
      user_gpio_set_value(GPIO_NUM_B2, 0);
    } else if (0 == uni_strcmp(setting->cmd, "SelfAbort")) {
      user_gpio_set_value(GPIO_NUM_B2, 1);
    } else if (0 == uni_strcmp(setting->cmd, "OutFive")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 5);
    } else if (0 == uni_strcmp(setting->cmd, "OutTen")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 10);
    } else if (0 == uni_strcmp(setting->cmd, "OutTwenty")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 20);
    } else if (0 == uni_strcmp(setting->cmd, "OutThirty")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 30);
    } else if (0 == uni_strcmp(setting->cmd, "OutForty")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 40);
    } else if (0 == uni_strcmp(setting->cmd, "OutFifty")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 50);
    } else if (0 == uni_strcmp(setting->cmd, "OutSixty")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 60);
    } else if (0 == uni_strcmp(setting->cmd, "OutSeven")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 70);
    } else if (0 == uni_strcmp(setting->cmd, "OutEight")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 80);
    } else if (0 == uni_strcmp(setting->cmd, "OutNine")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 90);
    } else if (0 == uni_strcmp(setting->cmd, "OutNineFive")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 95);
    } else if (0 == uni_strcmp(setting->cmd, "OutHund")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 100);
    } else if (0 == uni_strcmp(setting->cmd, "OutOFF")) {
      user_pwm_change_duty(PWM_NUM_1_A27, 0);
    } else if (0 == uni_strcmp(setting->cmd, "OV")) {
      user_uart_send(g_uart_buf[8].data, g_uart_buf[8].len);
    } else if (0 == uni_strcmp(setting->cmd, "CV")) {
      user_uart_send(g_uart_buf[9].data, g_uart_buf[9].len);
    } else if (0 == uni_strcmp(setting->cmd, "ASP")) {
      user_uart_send(g_uart_buf[10].data, g_uart_buf[10].len);
    } else if (0 == uni_strcmp(setting->cmd, "DSP")) {
      user_uart_send(g_uart_buf[11].data, g_uart_buf[11].len);
    } else if (0 == uni_strcmp(setting->cmd, "AS")) {
      user_uart_send(g_uart_buf[12].data, g_uart_buf[12].len);
    } else if (0 == uni_strcmp(setting->cmd, "DS")) {
      user_uart_send(g_uart_buf[13].data, g_uart_buf[13].len);
    } else {
      LOGT(TAG, "Unconcerned command: %s", setting->cmd);
    }
    user_player_reply_list_random(setting->reply_files);
  }
}

static void _register_event_callback(void) {
  user_event_subscribe_event(USER_CUSTOM_SETTING, _custom_setting_cb);
}

int hb_auto_gpio(void) {
  user_gpio_init();
  user_gpio_set_mode(GPIO_NUM_A25, GPIO_MODE_OUT);
  user_gpio_set_value(GPIO_NUM_A25, 1);
  user_gpio_set_mode(GPIO_NUM_A26, GPIO_MODE_OUT);
  user_gpio_set_value(GPIO_NUM_A26, 1);
  user_gpio_set_mode(GPIO_NUM_A28, GPIO_MODE_OUT);
  user_gpio_set_value(GPIO_NUM_A28, 0);
  user_gpio_set_mode(GPIO_NUM_B6, GPIO_MODE_OUT);
  user_gpio_set_value(GPIO_NUM_B6, 1);
  user_gpio_set_mode(GPIO_NUM_B7, GPIO_MODE_OUT);
  user_gpio_set_value(GPIO_NUM_B7, 1);
  user_gpio_set_mode(GPIO_NUM_B2, GPIO_MODE_OUT);
  user_gpio_set_value(GPIO_NUM_B2, 0);
  user_gpio_set_mode(GPIO_NUM_B3, GPIO_MODE_OUT);
  user_gpio_set_value(GPIO_NUM_B3, 1);
  user_pwm_init(PWM_NUM_1_A27, 50, false);
  user_pwm_start(PWM_NUM_1_A27, 0);
  user_uart_init(NULL);
  _register_event_callback();
  return 0;
}

